@@ -57,7 +57,7 @@ sub selectDb{
   }
 
   my $dbh=$self->{dbh};
-  $dbh->do(qq(
+  my $sth = $dbh->prepare(qq(
     CREATE TABLE DISTANCE(
       GENOME1     CHAR(255)    NOT NULL,
       GENOME2     CHAR(255)    NOT NULL,
@@ -65,6 +65,7 @@ sub selectDb{
       PRIMARY KEY(GENOME1,GENOME2)
     )) 
   );
+  $sth->execute();
 
   return 1;
 }
@@ -95,9 +96,9 @@ sub addDistances{
   my $dbh=$self->{dbh};
   my $numInserted=0;   # how many are going to be inserted?
 
-  my $baseInsertSql="INSERT INTO DISTANCE VALUES";
-  my @insertSQL=();
-  my $insertSQL=$baseInsertSql;
+  my $insert = $dbh->prepare( "INSERT INTO DISTANCE VALUES ( ?, ?, ? )" );
+  my $autocommit = $dbh->{AutoCommit};
+  $dbh->{AutoCommit} = 0; # begin a new transaction
   open(my $fh, "<", $distancesFile) or die "ERROR: could not read $distancesFile: $!";
   my $query="";
   while(<$fh>){
@@ -114,35 +115,14 @@ sub addDistances{
     
     next if(defined($self->findDistance($query,$subject)));
 
+    $insert->execute( $query, $subject, $distance );
+    if ( $dbh->err() ) {
+        die "Error: could not insert $distancesFile into the database.\n";
+    }
     $numInserted++;
-    $insertSQL.=qq( ("$query", "$subject", $distance), );
-    
-    # Avoid going over the SQLITE_MAX_COMPOUND_SELECT limit in sqlite3
-    if($numInserted % 99 == 0){
-      $insertSQL=~s/\s*,\s*$//; # remove whitespace and comma from the end
-      push(@insertSQL, $insertSQL);
-      $insertSQL=$baseInsertSql;
-    }
-  }
-
-  if($numInserted == 0){
-    return $numInserted;
-  }
-
-  # One last insert
-  if($insertSQL=~/VALUES.../){
-    $insertSQL=~s/\s*,\s*$//; # remove whitespace and comma from the end
-    push(@insertSQL, $insertSQL);
-    $insertSQL=$baseInsertSql;
-  }
-  
-  # Run through all the insert statements
-  for my $insertSQL(@insertSQL){
-    $dbh->do($insertSQL);
-    if($dbh->err()){
-      die "ERROR: could not insert $distancesFile into the database with query\n  $insertSQL\n  ".$dbh->err();
-    }
   }
+  $dbh->commit;
+  $dbh->{AutoCommit} = $autocommit;
 
   return $numInserted;
 }
@@ -155,10 +135,10 @@ sub findDistances{
   
   my $sth=$dbh->prepare(qq(SELECT GENOME2,DISTANCE 
     FROM DISTANCE 
-    WHERE GENOME1="$genome1"
+    WHERE GENOME1=?
     ORDER BY GENOME2
   ));
-  my $rv = $sth->execute() or die $DBI::errstr;
+  my $rv = $sth->execute( $genome1 ) or die $DBI::errstr;
   if($rv < 0){
     die $DBI::errstr;
   }
@@ -177,8 +157,8 @@ sub findDistance{
 
   my $dbh=$self->{dbh};
   
-  my $sth=$dbh->prepare(qq(SELECT DISTANCE FROM DISTANCE WHERE GENOME1="$genome1" AND GENOME2="$genome2"));
-  my $rv = $sth->execute() or die $DBI::errstr;
+  my $sth=$dbh->prepare(qq(SELECT DISTANCE FROM DISTANCE WHERE GENOME1=? AND GENOME2=?));
+  my $rv = $sth->execute( $genome1, $genome2 ) or die $DBI::errstr;
   if($rv < 0){
     die $DBI::errstr;
   }
@@ -238,22 +218,21 @@ sub toString_matrix{
 
 sub toString_tsv{
   my($self,$genome,$sortBy)=@_;
+  $sortBy||="abc";
+  $genome||=[];
+
   my $dbh=$self->{dbh};
 
+  # Index the genome array
+  my %genome;
+  $genome{_truncateFilename($_)}=1 for(@$genome);
+
   my $str="";
 
   my $sql=qq(
     SELECT GENOME1,GENOME2,DISTANCE
     FROM DISTANCE
   );
-  if(@$genome){
-    $sql.="WHERE \n";
-    for(@$genome){
-      $sql.="GENOME1 LIKE '$_%' OR \n";
-    }
-    $sql=~s/\s*OR\s*$//;
-    $sql.="\n";
-  }
   if($sortBy eq 'abc'){
     $sql.="ORDER BY GENOME1,GENOME2 ASC";
   } elsif($sortBy eq 'rand'){
@@ -268,6 +247,12 @@ sub toString_tsv{
 
   my %distance;
   while(my @row=$sth->fetchrow_array()){
+    # If the parameter was given to filter genome names,
+    # do it here.
+    my $truncatedName1 = _truncateFilename($row[0]);
+    my $truncatedName2 = _truncateFilename($row[1]);
+    next if(@$genome && (!$genome{$truncatedName1} || !$genome{$truncatedName2}));
+
     $_=~s/^\s+|\s+$//g for(@row); # whitespace trim
     $str.=join("\t",@row)."\n";
     $distance{$row[0]}{$row[1]}=$row[2];
@@ -278,26 +263,24 @@ sub toString_tsv{
 
 sub toString_phylip{
   my($self,$genome,$sortBy)=@_;
+  $sortBy||="abc";
+  $genome||=[];
   my $dbh=$self->{dbh};
 
+  # Index the genome array
+  my %genome;
+  $genome{_truncateFilename($_)}=1 for(@$genome);
+
   my $str="";
 
   # The way phylip is, I need to know the genome names
-  # a priori
+  # a priori. Get the genome names from the db.
   my @name;
   my $sql=qq(
     SELECT DISTINCT(GENOME1) 
     FROM DISTANCE 
+    ORDER BY GENOME1 ASC\n
   );
-  if(@$genome){
-    $sql.="WHERE \n";
-    for(@$genome){
-      $sql.="GENOME1 LIKE '$_%' OR \n";
-    }
-    $sql=~s/\s*OR\s*$//;
-    $sql.="\n";
-  }
-  $sql.="ORDER BY GENOME1 ASC\n";
   my $sth=$dbh->prepare($sql);
   my $rv=$sth->execute or die $DBI::errstr;
   if($rv < 0){
@@ -306,6 +289,11 @@ sub toString_phylip{
 
   my $maxGenomeLength=0;
   while(my @row=$sth->fetchrow_array()){
+    # If the parameter was given to filter genome names,
+    # do it here.
+    my $truncatedName = _truncateFilename($row[0]);
+    next if(@$genome && !$genome{$truncatedName});
+
     push(@name,$row[0]);
     $maxGenomeLength=length($row[0]) if(length($row[0]) > $maxGenomeLength);
   }
